//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "UploadManager.h"

typedef enum {
    
    uploadStatusUploading,
    uploadStatusUploaded,
    uploadStatusFailed
}   uploadStatus;

@interface UploadInfo : NSObject

@property (nonatomic, assign) long long totalBytesUploaded;
@property (nonatomic, assign) long long totalBytesToUploaded;
@property (nonatomic, assign) uploadStatus uploadStatus;

@end


@implementation UploadInfo

- (id)init {
    
    self = [super init];
    if (self) {
        
        _totalBytesToUploaded = 0;
        _totalBytesUploaded = 0;
        _uploadStatus = uploadStatusUploading;
    }
    return self;
}

@end

@implementation UploadObj

@end

@interface UploadManager ()

@property (nonatomic, strong) NSMutableArray *uploadArray;
@property (nonatomic, assign) int totalFilesDone;
@property (nonatomic, assign) int totalFilesToBeUploaded;
@property (nonatomic, assign) float progress;

@property (nonatomic, strong) UploadObj *uploadingFile;

@property (nonatomic, strong) NSOperationQueue *uploadOperationQueue;


@end

@implementation UploadManager

+ (UploadManager *)sharedInstance {
    static dispatch_once_t once;
    static UploadManager *sharedInstance;
    dispatch_once(&once, ^{sharedInstance = [[UploadManager alloc] init];});
    return sharedInstance;
}

- (id)init {
    
    self = [super init];
    
    if (self) {
        
        _uploadArray            = [NSMutableArray new];
        _uploadOperationQueue   = [[NSOperationQueue alloc] init];
        _uploadOperationQueue.maxConcurrentOperationCount = 1;
        
    }
    return self;
}

- (BOOL)isUploadInProgress {
    
    return [self.uploadArray count];
}

- (int)totalFilesToBeUploaded {
    
    return (int)[self.uploadArray count];
}


- (int)totalFilesDone {
    
    
    int filesDone = 0;
    
    for (UploadObj *file in self.uploadArray) {
        
        UploadInfo *info = file.uploadInfo;
        
        if (info.uploadStatus == uploadStatusUploaded || info.uploadStatus == uploadStatusFailed) {
            
            filesDone++;
        }
    }
    return filesDone;
}

- (void)addFileToUploadQueue:(UploadObj *)file {
    
    
    __block UploadInfo *info = [[UploadInfo alloc] init];
    info.totalBytesToUploaded = [[[NSFileManager defaultManager] attributesOfItemAtPath:file.fileName error:nil] fileSize];
    file.uploadInfo = info;
    [self.uploadArray addObject:file];
    
    __weak UploadManager *weakself = self;
    
    NSBlockOperation *uploadOperation = [NSBlockOperation blockOperationWithBlock:^{
        
        __block BOOL uploading = YES;
        // Intialize the PCUploadAPIParams object.
        PCUploadAPIParams *params = [[PCUploadAPIParams alloc] init];
        params.name = file.fileName;
        params.path = file.destinationFolder;
        
        weakself.uploadingFile = file;
        
        dispatch_async(dispatch_get_main_queue(), ^(void) {
            
            if ([weakself.delegate respondsToSelector:@selector(uploadManagerDidBeginUploadingFile:)]) {
                
                [weakself.delegate uploadManagerDidBeginUploadingFile:file];
            }
        });
        /*
         @Description :
        This API is used to  upload a file to a folder. Once the upload process is succeed/failed result will come with completion block.
         
         @Parameters :
         PCUploadAPIParams:
         * This class stores all the parameters required to upload  a file to a folder. It contains attributes such as path, name, size and checkSum.
         * For providing access token to SDK user have to logged In.
         * path, name, size and checkSum are required parameter to call this API.
         * filePath: local path of the file from where file has to be read. (required)
         * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
         * More information about parameters you will find in PCUploadAPIParams.h class.
         
         @CompletionHandler :
         This completion block gets called once the upload process is succeed/failed. completion block contains below parameter.
         PCAPIQueryResponse : Stores all the server response and error for this API.
         PCResource : This object stores metadata information of a file or a folder.
         progress : It will return totalBytesUploaded and totalBytesToUploaded.
         
         @To upload file to a folder, you must take the following steps:
         * Include VerizonCloudAPI.framework header file in your class.
         #import <VerizonCloudAPI/VerizonCloudAPI.h>
         * Initalize the PCUploadAPIParams class and provide the required parameter.
         * Initalize the PCAPISDK object and call the uploadFile:readFromPath:progress:withCompletionHandler: API.
         * Perform the  uploadFile:readFromPath:progress:withCompletionHandler: API call using the code sample below.
         * Get the result (succeed or failed) in completion block.
         * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
         */

        // Initilize the PCAPISDK and call the API.

        
        [[PCAPISDK sharedInstance] uploadFile:params
                                 readFromPath:file.filePath
                                     progress:^(PCUploadAPIState uploadState, long long totalBytesUploaded, long long totalBytesToBeUploaded) {
                                         dispatch_async(dispatch_get_main_queue(), ^(void) {
                                         
                                             info.totalBytesUploaded     = totalBytesUploaded;
                                             info.totalBytesToUploaded   = totalBytesToBeUploaded;
                                             
                                             if (totalBytesToBeUploaded > 0 && totalBytesUploaded > 0) {
                                                 
                                                 weakself.progress = ((float)totalBytesUploaded/(float)totalBytesToBeUploaded);
                                             }
                                             
                                             if ([weakself.delegate respondsToSelector:@selector(uploadManagerDidProgressedUploadingFile:progress:state:)]) {
                                                 
                                                 [weakself.delegate uploadManagerDidProgressedUploadingFile:file progress:weakself.progress state:uploadState];
                                             }
                                         });
                                     } withCompletionHandler:^(PCResource *metadata, PCAPIQueryResponse *response) {
                                         // Get the result in this block.
                                         dispatch_async(dispatch_get_main_queue(), ^(void) {

                                             if (response.urlResponse.statusCode == PCAPIResponseCodeResourceCreated) {
                                                 
                                                 info.uploadStatus = uploadStatusUploaded;
                                             } else {
                                                 
                                                 info.uploadStatus = uploadStatusFailed;
                                             }
                                             
                                             
                                             if (weakself.totalFilesDone >= weakself.totalFilesToBeUploaded) {
                                                 
                                                 [weakself.uploadArray removeAllObjects];
                                                 
                                                 UILocalNotification *localNotification = [[UILocalNotification alloc] init];
                                                 localNotification.alertBody = NSLocalizedString(@"Upload Done!", nil);
                                                 localNotification.soundName = UILocalNotificationDefaultSoundName;
                                                 
                                                 [[UIApplication sharedApplication] presentLocalNotificationNow:localNotification];
                                             }
                                             
                                             weakself.progress = 0.0;
                                             
                                             
                                             
                                             if ([weakself.delegate respondsToSelector:@selector(uploadManagerDidFinishUploadingFile:result:response:)]) {
                                                 
                                                 [weakself.delegate uploadManagerDidFinishUploadingFile:file result:metadata response:response];
                                             }
                                             
                                             uploading = NO;
                                         });
                                     }];
        
        while (uploading) {
            [[NSRunLoop currentRunLoop] runMode:NSDefaultRunLoopMode beforeDate:[NSDate distantFuture]];
        }
    }];
    
    
    [self.uploadOperationQueue addOperation:uploadOperation];
    
    if ([self.delegate respondsToSelector:@selector(uploadManagerDidAddFileToUploadQueue:)]) {
        
        [self.delegate uploadManagerDidAddFileToUploadQueue:file];
    }
}


- (void)cleanUpload {

    self.totalFilesDone = 0;
    self.totalFilesToBeUploaded = 0;
    self.progress = 0;
    self.uploadingFile = nil;
    [self.uploadOperationQueue cancelAllOperations];
    [self.uploadArray removeAllObjects];
}

@end
