//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "MetaDataViewController.h"
#import "UIViewController+Alert.h"
#import "UIViewController+RightBarButtonAction.h"
#import "UIViewController+ResourceActions.h"
#import "UIViewController+Spinner.h"
#import "NSArray+Addition.h"
#import "UploadManager.h"
#import "ProgressView.h"

@interface FileGridViewController (Protected)

@property (nonatomic, strong) NSString *emptyViewMessage;
@property (nonatomic, retain) UIRefreshControl *refreshControl;

- (void)refreshView;
- (void)alertController:(UIAlertController *)alertController
     addActionWithTitle:(NSString *)title
                  style:(UIAlertActionStyle)style
              andAction:(void (^)(UIAlertAction * _Nonnull action))handler;

@end


@interface MetaDataViewController () <UploadDelegate>

@property (weak, nonatomic) IBOutlet ProgressView *progressView;

@end

@implementation MetaDataViewController

- (void)reloadData {
    if (self.rootResource) {
        self.title = self.rootResource.name;
    } else {
        self.title = NSLocalizedString(@"ROOT", nil);
    }
    
    // Intialize the PCMetaDataAPIParams object.
    PCMetaDataAPIParams *params = [[PCMetaDataAPIParams alloc] init];
    params.path = self.rootResource.fullPath;
    
    __weak MetaDataViewController *blocksafeSelf = self;
    /*
     @Description :
        This API is used to fetch metadata of a folder. Once API call is successful, PCMetadata object returned with updated value. Result will come
        with completion block. For providing access token to SDK user have to logged In.
     
     @ReturnType : PCAPI object using this return type we can cancel the ongoing request.
     
     @Parameters :
     PCMetaDataAPIParams:
     * This class stores all the parameters required to fetch metadata of a folder. It contains attributes such as path, includeDeleted, sort, start, count and filter.
     *  For providing access token to SDK user have to logged In.
     * path is required parameter to call this API.
     * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
     * includeDeleted, sort, start, count, filter are optional parameter to call this API.
     * More information about parameters you will find in PCMetaDataAPIParams.h class.
     
     @CompletionHandler -
     This completion block gets called once the fetch process is succeed/failed. completion block contains below parameter.
     PCMetadata : This object stores array of respective subFolders and subFiles, in which each array object will have the PCResource object in it.
     PCAPIQueryResponse : Stores all the server response and error for this API.
     
     @To fetch metadata of a folder, you must take the following steps:
     * Include VerizonCloudAPI.framework header file in your class.
     #import <VerizonCloudAPI/VerizonCloudAPI.h>
     * Initalize the PCMetaDataAPIParams class and provide the required parameter.
     * Initalize the PCAPISDK object and call the fetchMetadata:withCompletionHandler: API.
     * Perform the fetchMetadata:withCompletionHandler: API call using the code sample below.
     * Get the result (succeed or failed) in completion block.
     * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
     */
    
    // Initilize the PCAPISDK and call the API.
    [[PCAPISDK sharedInstance] fetchMetadata:params withCompletionHandler:^(PCMetadata *metadata, PCAPIQueryResponse *response) {
        // Get the result in this block.
        [blocksafeSelf hideSpinner];
        [blocksafeSelf.refreshControl endRefreshing];

        switch (response.urlResponse.statusCode) {
            case PCAPIResponseCodeOK: {
                
                blocksafeSelf.dataSource = [metadata.subFolders arrayByAddingObjectsFromArray:metadata.subFiles];
                [blocksafeSelf refreshView];
            }
                break;
            case PCAPIResponseCodeUnauthorized:
                [self showUnAuthorizedAlert];
                break;
            case PCAPIResponseCodeServiceUnavailable:
                [self showServerUnavailableAlert];
                break;
            default:
                [self showOperationFailedAlert:NSLocalizedString(@"Metadata fetch", nil)];
                break;
        }       
    }];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    if (self.rootResource) {
        
        self.title = self.rootResource.name;
    } else {
        
        self.title = NSLocalizedString(@"ROOT", nil);
    }
    
    self.emptyViewMessage = NSLocalizedString(@"Folder is empty", nil);
    
    [self showSpinner];
    [self.refreshControl beginRefreshing];

    [self reloadData];
    

}

- (void)viewWillAppear:(BOOL)animated {

    [super viewWillAppear:animated];
    
    [UploadManager sharedInstance].delegate = self;
    
    if ([UploadManager sharedInstance].isUploadInProgress) {
        
        self.progressView.hidden = NO;
        self.progressView.progressBar.progress = [UploadManager sharedInstance].progress;
        self.progressView.subLabel.text = [NSString stringWithFormat:@"(%d/%d)", [UploadManager sharedInstance].totalFilesDone, [UploadManager sharedInstance].totalFilesToBeUploaded];
        self.progressView.imageView.image = [UploadManager sharedInstance].uploadingFile.thumbnailImage;
        self.progressView.mainLabel.text = [NSString stringWithFormat:NSLocalizedString(@"Uploading file %@", nil), [UploadManager sharedInstance].uploadingFile.fileName];


    } else {
        self.progressView.hidden = YES;
    }
}

#pragma mark -
#pragma Upload delegate

- (void)uploadManagerDidAddFileToUploadQueue:(UploadObj *)file {

    self.progressView.hidden = NO;
    self.progressView.subLabel.text = [NSString stringWithFormat:@"(%d/%d)", [UploadManager sharedInstance].totalFilesDone, [UploadManager sharedInstance].totalFilesToBeUploaded];
}

- (void)uploadManagerDidBeginUploadingFile:(UploadObj *)file {

    self.progressView.mainLabel.text = [NSString stringWithFormat:NSLocalizedString(@"Uploading file %@", nil), file.fileName];
    self.progressView.progressBar.progress = 0.0;
    self.progressView.imageView.image = file.thumbnailImage;
}

- (void)uploadManagerDidProgressedUploadingFile:(UploadObj *)file progress:(CGFloat)progress state:(PCUploadAPIState)state {
   
    switch (state) {
        case PCUploadAPIStateInitiatingUpload:
            self.progressView.mainLabel.text = [NSString stringWithFormat:NSLocalizedString(@"Initiating upload for file %@", nil), file.fileName];
            break;
        case PCUploadAPIStateCommitingUpload:
            self.progressView.mainLabel.text = [NSString stringWithFormat:NSLocalizedString(@"Finalizing upload for file %@", nil), file.fileName];
            break;
        default:
            self.progressView.mainLabel.text = [NSString stringWithFormat:NSLocalizedString(@"Uploading file %@", nil), file.fileName];
            break;
    }
    
    self.progressView.progressBar.progress = progress;
}

- (void)uploadManagerDidFinishUploadingFile:(UploadObj *)file result:(PCResource *)metadata response:(PCAPIQueryResponse *)response {

    
    if ([metadata.parentPath isEqualToString:self.rootResource.fullPath]) {
        
        [self didFinishDoingRightBarButtonAction:uploadFileRightBarAction :metadata :response];
    }
    

    self.progressView.progressBar.progress = 0;
    self.progressView.mainLabel.text = nil;
    self.progressView.imageView.image = nil;

    if ([UploadManager sharedInstance].isUploadInProgress) {
        
        self.progressView.subLabel.text = [NSString stringWithFormat:@"(%d/%d)", [UploadManager sharedInstance].totalFilesDone, [UploadManager sharedInstance].totalFilesToBeUploaded];
    } else {
    
        self.progressView.hidden = YES;
    }
}


- (id)dataSourceForRightBarButtonAction:(rightBarAction)action {
    id dataSource = [super dataSourceForRightBarButtonAction:action];
    switch (action) {
        case uploadFileRightBarAction:
        case createFolderRightBarAction: {
            return self.rootResource.fullPath;
        }
            break;
        default:
            return dataSource;
            break;
    }
}

@end
